/***********************************************************************
 * Copyright 2016, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:        rskmimpl_migration_path_ds.sas
 * PURPOSE:     Macro to change absolute directory path in the configuration datasets during MIP migration  
 *
 * PARAMETERS: 
 *     THESE PARAMETERS ARE REQUIRED
 *     CURR_DIR :  The current deployed fully qualified path, 
 *                 such as:  \local\install\SASConfig\Lev1\AppData\SASModelImplementationPlatform.
 *     OLD_PATH :  The source environment, i.e., the original environment work group directory structure.  
 *                 The program will find this string and replace it with NEW_PATH.
 *     NEW_PATH :  The target environment, i.e., the new environment work group directory structure.  
 *                 The program will replace with this string. 
 *     TMP_DIR  :  A temporary directory under SAS Config. This is where the sas program will reside, 
 *                 after the tokens have been substituted, and the program can use as a temporary directory for misc files.
 *     WORKGROUP_NAME : Workgroup name
 *     OLD_APPDATA_MIP_PATH: The source environment of MIP AppData directory
 *     NEW_APPDATA_MIP_PATH: The target environment of MIP AppData directory
 *     An external sas dataset (named config_file_list) needs to be put in TMP_DIR and hold information about the list of config
 *     datasets that need to be changed. 
 ***********************************************************************/

%macro rskmimpl_migration_path_ds(CURR_DIR=,OLD_PATH=,NEW_PATH=,TMP_DIR=, WORKGROUP_NAME=, old_appdata_mip_path=, new_appdata_mip_path=);

   /* check necessary arguments */
   %if %sysevalf(%superq(CURR_DIR)=,boolean) %then %do;
      %put ERROR: The CURR_DIR parameter is required for the change_path macro.;
      %return;
   %end;
   %if %sysevalf(%superq(OLD_PATH)=,boolean) %then %do;
      %put ERROR: The OLD_PATH parameter is required for the change_path macro.;
      %return;
   %end;
   %if %sysevalf(%superq(NEW_PATH)=,boolean) %then %do;
      %put ERROR: The NEW_PATH parameter is required for the change_path macro.;
      %return;
   %end;
   %if %sysevalf(%superq(TMP_DIR)=,boolean) %then %do;
      %put ERROR: The TMP_DIR parameter is required for the change_path macro.;
      %return;
   %end;
   %if %sysevalf(%superq(old_appdata_mip_path)=,boolean) %then %do;
      %put ERROR: The OLD_APPDATA_MIP_PATH parameter is required for the change_path macro.;
      %return;
   %end;
   %if %sysevalf(%superq(new_appdata_mip_path)=,boolean) %then %do;
      %put ERROR: The NEW_APPDATA_MIP_PATH parameter is required for the change_path macro.;
      %return;
   %end;
   
   data _null_;
      curr_dir=prxchange('s/(\\|\/)$//',1,"&CURR_DIR.");
      call symputx("CURR_DIR",curr_dir);
      old_path=prxchange('s/(\\|\/)$//',1,"&OLD_PATH.");
      call symputx("OLD_PATH",old_path);
      new_path=prxchange('s/(\\|\/)$//',1,"&NEW_PATH.");
      call symputx("NEW_PATH",new_path);
      tmp_dir=prxchange('s/(\\|\/)$//',1,"&TMP_DIR.");
      call symputx("TMP_DIR",tmp_dir);
      old_appdata_mip_path=prxchange('s/(\\|\/)$//',1,"&old_appdata_mip_path.");
      call symputx("old_appdata_mip_path",old_appdata_mip_path);
      new_appdata_mip_path=prxchange('s/(\\|\/)$//',1,"&new_appdata_mip_path.");
      call symputx("new_appdata_mip_path",new_appdata_mip_path);
   run;

   options noserror nomerror;

   /* set up configuration migration change information */
   %local target_data_sets column_names reference_vars unique_targets;
   %if %sysfunc(fileexist("&TMP_DIR./config_file_list.csv")) %then %do;
      data config_file_list;
        length target_data_set $ 128 column_name $ 128 reference_var $ 2048;
        infile "&TMP_DIR./config_file_list.csv" dsd missover firstobs=2;
        input target_data_set : $128. column_name : $128. reference_var : $2048.;
     run;

      libname tmp "&TMP_DIR.";
      proc sql noprint; 
          select target_data_set, column_name, reference_var into :target_data_sets separated by ' ',
          :column_names separated by ' ', :reference_vars separated by ' ' 
          from config_file_list; 
      quit;

      proc sql noprint; 
          select unique(target_data_set) into :unique_targets separated by ' ' from config_file_list; 
      quit;
   %end;
   %else %do;
      %put ERROR: config_file_list.csv does not exist.;
     %return;
   %end;

    /***********************************************************************************/
    /*****Part 1: list all targeted sas dataset names with full path under CURR_DIR*****/
    /***********************************************************************************/
    
    /* call windows or linux commands to find the config sas datasets */
    %local target_cnt winstr linuxstr target;
    %let target_cnt=1;
    %let winstr=%str();
    %let linuxstr=%str();
    %do %while (%scan(&unique_targets.,&target_cnt.,' ') ne );
        %let target=%scan(&unique_targets.,&target_cnt.,' ');
        %let winstr=&winstr. &CURR_DIR\*&target..sas7bdat;
        %if &target_cnt.=1 %then %do;
        %let linuxstr=-name *&target..sas7bdat;
        %end; %else %do;
        %let linuxstr=&linuxstr -o -name *&target..sas7bdat;
        %end;
        %let target_cnt=%eval(&target_cnt.+1);
    %end;

    %if &sysscp eq WIN %then %do;
        options noxwait;
        x "cmd /C dir &winstr /b /s >&TMP_DIR.\&WORKGROUP_NAME._list.txt";
    %end;
    %else %do;
        x "find ""&CURR_DIR."" &linuxstr>&TMP_DIR./&WORKGROUP_NAME._list.txt";
    %end;

    /* save the dataset names with path into filenames dataset */
    data filenames;
        infile "&TMP_DIR./&WORKGROUP_NAME._list.txt";
        length filenames $2048.;
        input filenames $;
    run;
    
    /* do not change files in the backup folder */
    data filenames;
        set filenames;
        where prxmatch('/(\/|\\)backup\_\d{4}\-\d{2}\-\d{2}\-\d{2}\.\d{2}/',filenames)=0;
    run;

    proc sql noprint; 
        select count(*) into: filetot from filenames; 
    quit; 

    /* check whether all files and columns are found */
    %let no_notes_flag=1;

    %if &filetot.>0 %then %do;
        proc sort data=filenames nodup; 
            by filenames; 
        run;

        /**************************************************************************/
        /*****      Part 2: change old path to new path or reference_var      *****/
        /**************************************************************************/
        %local count target_data_set column_name reference_var;
        %let count=1;
      
        %do %while (%scan(&target_data_sets.,&count.,' ') ne );
            %let target_data_set=%scan(&target_data_sets.,&count.,' ');
            %let column_name=%scan(&column_names.,&count.,' ');
            %let reference_var=%scan(&reference_vars.,&count.,' ');

            /* find datasets which match target dataset name, can be under different directories */
            data target_lib;
                set filenames;
            %if &sysscp eq WIN %then %do;
                   nameindex=index(filenames,"\&target_data_set."||".sas7bdat");
            %end;
            %else %do;
               nameindex=index(filenames,"/&target_data_set."||".sas7bdat");
            %end;
                if nameindex>0 then do;
                    libpath=substr(filenames,1,nameindex);
                    output target_lib;
                end;
            run;

            proc sql noprint; 
                select count(*) into :filecount
                from target_lib; 
            quit;

            %if &filecount.=0 %then %do;
                %let no_notes_flag=0;
            %end;
            %else %do;
                %do i=1 %to &filecount.;
                    data _null_;
                  set target_lib;
                  if _n_=&i. then do;
                          call symput("libpath",strip(libpath));
                  end;
               run;
                    
                    libname tpath "&libpath.";
                    %let target=tpath.&target_data_set.; 

                    /* check whether column exists */
                    %local dsid col_exist replace;
                    %let dsid=%sysfunc(open(&target.));
                    %if %sysfunc(varnum(&dsid,&column_name))=0 %then %do;
                        %let col_exist=0;
                        %put WARNING: Var &column_name. not exists in &libpath.&target_data_set..sas7bdat; 
                        %let no_notes_flag=0;
                    %end; 
                    %else %do; 
                        %let col_exist=1; 
                    %end;
                    %let rc=%sysfunc(close(&dsid));

                    %if &col_exist.=1 %then %do;
                        /* change old Path to new Path or reference_var based on 
                           specification in sas_file_migration_config dataset */
                        %if %qupcase(&reference_var.) ne NEW %then %do; 
                            %let replace=&reference_var.; 
                        %end;
                        %else %do; 
                            %let replace=&NEW_PATH.; 
                        %end; 
                        data &target.;
                            set &target.;
                            /*Change work group path first*/
                            &column_name.=tranwrd(&column_name.,"&OLD_PATH.","&replace.");
                            
                            /*Change MIP AppData path*/
                            &column_name.=tranwrd(&column_name.,"&old_appdata_mip_path.","&new_appdata_mip_path.");
                            
                            /*Handle _execution_config_summary data set for migration 2.x->3.x*/
                            %if %lowcase(&target_data_set) eq _execution_config_summary %then %do;
                               if NAME in ("LIB_CUSTOM_DATA", "LIB_WORK_GROUP") then do;
                                   dir_path=tranwrd(dir_path, '&MI_ROOT', "&NEW_PATH");
                               end;                            
                            %end;
                        run;
                    %end;
                %end;
            %end;
            %let count=%eval(&count.+1);
        %end;
    %end;
    %else %do;
        %put WARNING: No target files was found;
        %let no_notes_flag=0;
    %end;
    %if &no_notes_flag=1 %then %do;
        %put NOTE: Task completed with all files with old path changed;
    %end;

%mend rskmimpl_migration_path_ds;
