/***********************************************************************
 * Copyright 2015, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:          rskmimpl_aa_create_scen_chg_asof
 *
 * PURPOSE:       Create the scenario data by shifting date from the as-of date in the run1
 *                to the as-of date in the run2. This macro takes the economic data sets as input
 *                and create HPRISK required format data sets: history, current and scenarios data sets.
 *
 * PARAMETERS: 
 *              rskmimpl_aa_run1_econ_lib 
 *                  <required> - Specify the economic data library of the run1
 *              rskmimpl_aa_run2_econ_lib 
 *                  <required> - Specify the economic data library of the run2
 *              rskmimpl_aa_out_lib  
 *                  <required> - Specifies the output library
 *              rskmimpl_aa_run1_run_date  
 *                  <required> - Specifies the as-of date of the run1 
 *              rskmimpl_aa_run1_scenarios 
 *                  <optional> - Specifies scenarios in the run1 to be kept when creating 
 *                               the new scenario data set. 
 *                  Note: It can contains multiple scenarios. But they need to be in JSON format. 
 *              rskmimpl_aa_run2_run_date  
 *                  <required> - Specifies the as-of date of the run2 
 *              rskmimpl_aa_run2_scenarios 
 *                  <optional> - Specifies scenarios in the run1 to be kept when creating 
 *                               the new scenario data set. 
 *                  Note: It can contains multiple scenarios. But they need to be in JSON format. 
 *              rskmimpl_aa_rename_scenario 
 *                  <optional> - Specifies a scenario name to which the scenario in the run1 and run2 is renamed.
 *                  if not specified, the scenario in the run1 and run2 will not be renamed.
 *                  
 *                  Note: this argument is only valid and used when rskmimpl_aa_run1_scenarios and rskmimpl_aa_run2_scenarios
 *                        only contain one scenario.
 *              rskmimpl_aa_interval 
 *                  <Required> - Specifies the interval
 *              rskmimpl_aa_maxhorizon 
 *                  <Required> - Specifies the maximum horizon that the scenario data set cotains.
 *              rskmimpl_aa_error_abort  
 *                  <optinal>  - Abort the current SAS process if errors occur when it is "YES". 
 *                               By default it is "NO".
 *              rskmimpl_aa_base_env 
 *                  <optional> - Specifies the risk environment to check defined foreign exchange rate risk factors.
 *
 *  EXAMPLE: %rskmimpl_aa_create_scen_chg_asof(
 *                                             rskmimpl_aa_run1_econ_lib=mipecon1,
 *                                             rskmimpl_aa_run2_econ_lib=mipecon2,
 *                                             rskmimpl_aa_out_lib=mipecon,
 *                                             rskmimpl_aa_run1_run_date='01Dec2012'd,
 *                                             rskmimpl_aa_run1_scenarios=["adverse"],
 *                                             rskmimpl_aa_run2_run_date='01Dec2013'd,
 *                                             rskmimpl_aa_run2_scenarios=["base"],
 *                                             rskmimpl_aa_rename_scenario=attribution_analysis,
 *                                             rskmimpl_aa_interval=month,       
 *                                             rskmimpl_aa_maxhorizon=24     
 *                                             );
 ***********************************************************************/
%macro rskmimpl_aa_create_scen_chg_asof(rskmimpl_aa_run1_econ_lib=,
                                        rskmimpl_aa_run2_econ_lib=,
                                        rskmimpl_aa_out_lib=,
                                        rskmimpl_aa_run1_run_date=,
                                        rskmimpl_aa_run1_scenarios=,
                                        rskmimpl_aa_run2_run_date=,
                                        rskmimpl_aa_run2_scenarios=,
                                        rskmimpl_aa_interval=Month,
                                        rskmimpl_aa_maxhorizon=,
                                        rskmimpl_aa_error_abort=yes,
                                        rskmimpl_aa_rename_scenario=,
                                        rskmimpl_aa_base_env=mipenv.risk_env
                                        );
    %local econ_ds_list;
    
    %let rskmimpl_aa_rename_scenario=%bquote(%lowcase(&rskmimpl_aa_rename_scenario));

    proc sql noprint;
        select memname into :econ_ds_list separated by " "
        from sashelp.vtable
        where libname="%upcase(&rskmimpl_aa_run1_econ_lib)";
    quit;

    %if %length(&econ_ds_list) eq 0 %then %do;
        %let msg=%sysfunc(SASMSG(sashelp.rskmimpl, rskmimpl.aa.create.econ.asof.chg.nodata, N, %upcase(&rskmimpl_aa_run1_econ_lib)));
        %rskmimpl_print_msg(msg=%str(&msg), msg_type=error, msg_step=&msg_step, error_abort=&rskmimpl_aa_error_abort);
        %return;
    %end;

    proc datasets library=&rskmimpl_aa_out_lib kill nolist;
    run; 

    %local run1_scenario_ds run1_num_scenarios;
    %if not %sysevalf(%superq(rskmimpl_aa_run1_scenarios)=,boolean) %then %do;
        %let run1_scenario_ds=run1_scenarios;
        %rskmimpl_read_scenarios_into_ds(rskmimpl_scenarios=%superq(rskmimpl_aa_run1_scenarios),
                                         rskmimpl_output_ds=&run1_scenario_ds,
                                         rskmimpl_num_scenarios_mac_var=run1_num_scenarios,
                                         rskmimpl_option=lowcase
                                         );
    %end;
  
    %local _SCENARIO_NAME_4_convert_macro_;
    %if &run1_num_scenarios=1 %then %do;
        data _null_;
            set &run1_scenario_ds;
            call symput("_SCENARIO_NAME_4_convert_macro_", strip(scenario_name));
        run;
    %end;
    %else %let _SCENARIO_NAME_4_convert_macro_ = &rskmimpl_aa_rename_scenario;

    %rskmimpl_convert_econdata(MODIMP_ECONDATA_LIB=&rskmimpl_aa_run1_econ_lib,
                               MODIMP_RUNASOFDATE=&rskmimpl_aa_run1_run_date,
                               MODIMP_MAXHORIZON=&rskmimpl_aa_maxhorizon,
                               MODIMP_ECONDATA_INTERVAL=&rskmimpl_aa_interval                      ,
                               MODIMP_OUTPUT_ECONDATA_HISTORY=&rskmimpl_aa_out_lib..Econdata_History,
                               MODIMP_OUTPUT_ECONDATA_CURRENT=&rskmimpl_aa_out_lib..Econdata_Current,
                               MODIMP_OUTPUT_ECONDATA_SCENARIOS=&rskmimpl_aa_out_lib..Econdata_Scenarios ,
                               MODIMP_OUTPUT_READVARS_DATASET=Econdata_Readvars,
                               MODIMP_OUTPUT_READREFS_DATASET=Econdata_Readrefs,
                               MODIMP_SCENARIO_NAME=%superq(_SCENARIO_NAME_4_convert_macro_),
                               MODIMP_ABORT_ON_ERRORS=&rskmimpl_aa_error_abort,
                               MODIMP_BASE_ENV=&rskmimpl_aa_base_env,
                               MODIMP_RF_DESC_DS= 
                               );
                               
     /*Shift date in econdata_history by (rskmimpl_aa_run2_run_date-rskmimpl_aa_run1_run_date)*/   
     %if %sysfunc(exist(&rskmimpl_aa_out_lib..Econdata_History)) %then %do;
         %rskmimpl_aa_shift_date_history(rskmimpl_aa_history_ds=&rskmimpl_aa_out_lib..Econdata_History,
                                         rskmimpl_aa_run2_econ_lib=&rskmimpl_aa_run2_econ_lib,
                                         rskmimpl_aa_run1_run_date=&rskmimpl_aa_run1_run_date,
                                         rskmimpl_aa_run2_run_date=&rskmimpl_aa_run2_run_date,
                                         rskmimpl_aa_interval=&rskmimpl_aa_interval
                                         );    
     %end;
     
     /*Subset scenarios and rename scenario in Econdata_Scenarios*/
     data &rskmimpl_aa_out_lib..Econdata_Scenarios;
         set &rskmimpl_aa_out_lib..Econdata_Scenarios;

         scenario_name=lowcase(scenario_name);
         %if &run1_num_scenarios>0 %then %do;
             if _n_=1 then do;
                 declare hash _keep_scens(dataset:"&run1_scenario_ds"); 
                 _keep_scens.defineKey("scenario_name");
                 _keep_scens.defineData("scenario_name");
                 _keep_scens.defineDone();
             end;
                    
             if _keep_scens.find() ne 0 then delete; 
         %end;

         %if &run1_num_scenarios=1 and not %sysevalf(%superq(rskmimpl_aa_rename_scenario)=,boolean) %then %do;
              retain _rename_scen_;
              if _n_=1 then _rename_scen_=symget("rskmimpl_aa_rename_scenario");
              scenario_name=_rename_scen_;
              drop _rename_scen_;
         %end;
     run;
      
%mend rskmimpl_aa_create_scen_chg_asof;


%macro rskmimpl_aa_shift_date_history(rskmimpl_aa_history_ds=,
                                      rskmimpl_aa_run2_econ_lib=,
                                      rskmimpl_aa_run1_run_date=,
                                      rskmimpl_aa_run2_run_date=,
                                      rskmimpl_aa_interval=
                                      );

    /*Shift date in history data by (rskmimpl_aa_run2_run_date-rskmimpl_aa_run1_run_date)*/   
    %local i econ_ds num_econ_ds min_date_in_a_ds history_min_date ds_w_min_date history_horizon_ds nobs_history_horizon_ds nobs_history_ds;

    %let nobs_history_ds = 0;
    data _null_;
        call symputx("nobs_history_ds", nobs);
        stop;
        set &rskmimpl_aa_history_ds nobs=nobs;
    run;
    %if &nobs_history_ds = 0 %then %return;

    %let econ_ds_list = ;
    proc sql noprint;
        select memname into :econ_ds_list separated by " "
        from sashelp.vtable
        where libname="%upcase(&rskmimpl_aa_run2_econ_lib)";
    quit;
    %let num_econ_ds=&SQLOBS;

    %let history_min_date=&rskmimpl_aa_run2_run_date;
    %do i=1 %to &num_econ_ds;
        %let econ_ds=%scan(&econ_ds_list, &i, %str( ));
        proc sql noprint;
           select min(date) into :min_date_in_a_ds trimmed
           from &rskmimpl_aa_run2_econ_lib..&econ_ds
           where date < &history_min_date;
        quit;
             
        %if "&min_date_in_a_ds" ne "." %then %do;
           %let history_min_date=&min_date_in_a_ds;
           %let ds_w_min_date=&rskmimpl_aa_run2_econ_lib..&econ_ds;
        %end;
    %end;
         
    %let nobs_history_horizon_ds = 0;
    %if not %sysevalf(%superq(ds_w_min_date)=,boolean) %then %do;
        %let history_horizon_ds= _racsca_history_horizon_;
        proc sort data=&ds_w_min_date(keep=date where=(date<&rskmimpl_aa_run2_run_date))
                  out=&history_horizon_ds nodupkeys;
            by descending date;
        run;
             
        data &history_horizon_ds;
            set &history_horizon_ds nobs=nobs;
            if _n_=1 then call symputx("nobs_history_horizon_ds", nobs);
            _merge_horizon_tmp_ = -_n_;
        run;   
    %end;

    %if &nobs_history_horizon_ds = 0 %then %do;
         data &rskmimpl_aa_history_ds;
             set &rskmimpl_aa_history_ds;

             retain _asof_chg_dist_;
             if _n_=1 then _asof_chg_dist_=intck("&rskmimpl_aa_interval", &rskmimpl_aa_run1_run_date, &rskmimpl_aa_run2_run_date);

             if _asof_chg_dist_ ne 0 then do;
                 if date = intnx("&rskmimpl_aa_interval", date, 0, 'end') then do;
                     date=intnx("&rskmimpl_aa_interval", date, _asof_chg_dist_, 'end');
                 end;
                 else do;
                     date=intnx("&rskmimpl_aa_interval", date, _asof_chg_dist_, 'same');
                 end;
             end;
             drop _asof_chg_dist_;
         run;
    %end;
    %else %do;
         %if &nobs_history_horizon_ds < &nobs_history_ds %then %do;
             data &history_horizon_ds;
                 set &history_horizon_ds end=eof;
                 length _alignment $4;

                 output;
                 if eof then do;
                    if date = intnx("&rskmimpl_aa_interval", date, 0, 'end') then _alignment = "end";
                    else _alignment = "same";

                    do _i=1 to (&nobs_history_ds - &nobs_history_horizon_ds);
                        date=intnx("&rskmimpl_aa_interval", date, -1, _alignment);
                        _merge_horizon_tmp_=_merge_horizon_tmp_-1;
                        output;
                    end;
                 end;

                 drop _i _alignment;
             run; 
         %end;

         data &rskmimpl_aa_history_ds;
             set &rskmimpl_aa_history_ds;

             _merge_horizon_tmp_ = -&nobs_history_ds -1 + _n_;
                 
             if _n_=1 then do;
               declare hash _his_horizon(dataset:"&history_horizon_ds");
               _his_horizon.definekey('_merge_horizon_tmp_');
               _his_horizon.definedata('date');
               _his_horizon.definedone();
             end;
                 
             if _his_horizon.find() eq 0; 
             drop _merge_horizon_tmp_;
         run;
    %end;
         
%mend rskmimpl_aa_shift_date_history;
