/***********************************************************************
 * Copyright 2015, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:          rskmimpl_aa_create_scen_chg_econ
 *
 * PURPOSE:       Create the history, current and scenario data sets by swapping in the specified 
 *                economic variables from the run2.
 *
 * PARAMETERS: 
 *              rskmimpl_aa_run1_econ_lib 
 *                  <required> - Specify the economic data library of the run1
 *              rskmimpl_aa_run2_econ_lib 
 *                  <required> - Specify the economic data library of the run2
 *              rskmimpl_aa_prev_grp_lib  
 *                  <required> - Specifies the library that contains the history, current and
 *                               scenario data that need to be swapped in economic data from the run2
 *              rskmimpl_aa_curr_grp_lib  
 *                  <required> - Specifies the output library 
 *              rskmimpl_aa_swap_econ_vars 
 *                  <required> - Specifies the economic variables to be swapped 
 *
 *  EXAMPLE: %rskmimpl_aa_create_scen_chg_econ(
 *                                             rskmimpl_aa_run1_econ_lib=mipecon1,
 *                                             rskmimpl_aa_run2_econ_lib=mipecon2,
 *                                             rskmimpl_aa_prev_grp_lib=mipaad1,
 *                                             rskmimpl_aa_curr_grp_lib=mipaad2,
 *                                             rskmimpl_aa_swap_econ_vars=pmms15yr pmms30yr
 *                                             );
 ***********************************************************************/
%macro rskmimpl_aa_create_scen_chg_econ(rskmimpl_aa_run1_econ_lib=,
                                        rskmimpl_aa_run2_econ_lib=,
                                        rskmimpl_aa_prev_grp_lib=,
                                        rskmimpl_aa_curr_grp_lib=,
                                        rskmimpl_aa_swap_econ_vars=
                                        );
    
    proc copy in=&rskmimpl_aa_prev_grp_lib out=&rskmimpl_aa_curr_grp_lib;
        select %if %sysfunc(exist(&rskmimpl_aa_prev_grp_lib..econdata_history)) %then econdata_history;
               econdata_current econdata_scenarios;
    run;

    %if %sysevalf(%superq(rskmimpl_aa_swap_econ_vars)=,boolean) %then %return;

    %local swap_econ_vars_quoted num_obs i risk_factor num_swap_econ_vars;
    %let swap_econ_vars_quoted=%unquote(%nrbquote(')%qsysfunc(prxchange(s/\s+/%nrbquote(',')/oi,-1,%upcase(&rskmimpl_aa_swap_econ_vars)))%nrbquote('));
    %let num_swap_econ_vars = %sysfunc(countw(&rskmimpl_aa_swap_econ_vars));
    
    data _rskmimpl_aa_rf_to_drop;
        set &rskmimpl_aa_run1_econ_lib..econdata_readvars(keep=name group);
        where upcase(name) in (&swap_econ_vars_quoted) 
              or upcase(group) in (&swap_econ_vars_quoted);
        name=upcase(name);
        keep name;
    run;
    
    data _rskmimpl_aa_rf_to_drop_base;
        set &rskmimpl_aa_run1_econ_lib..base_env_readvars(keep=name group);
        where upcase(name) in (&swap_econ_vars_quoted) 
              or upcase(group) in (&swap_econ_vars_quoted);
        name=upcase(name);
        keep name;
    run;
    
    proc append base=_rskmimpl_aa_rf_to_drop data=_rskmimpl_aa_rf_to_drop_base;
    run;
   
    proc sort data=_rskmimpl_aa_rf_to_drop nodupkey;
        by name;
    run;

    data _rskmimpl_aa_rf_to_swap;
        set &rskmimpl_aa_run2_econ_lib..econdata_readvars(keep=name group);
        where upcase(name) in (&swap_econ_vars_quoted) 
              or upcase(group) in (&swap_econ_vars_quoted);
        name=upcase(name);
        keep name;
    run;

    data _rskmimpl_aa_rf_to_swap_base;
        set &rskmimpl_aa_run2_econ_lib..base_env_readvars(keep=name group);
        where upcase(name) in (&swap_econ_vars_quoted) 
              or upcase(group) in (&swap_econ_vars_quoted);
        name=upcase(name);
        keep name;
    run;
    
    proc append base=_rskmimpl_aa_rf_to_swap data=_rskmimpl_aa_rf_to_swap_base;
    run;
    
    proc sort data=_rskmimpl_aa_rf_to_swap nodupkey;
        by name;
    run;

    /*Swap history data*/
    %rskmimpl_aa_swap_history_current(rskmimpl_aa_run2_econ_lib=&rskmimpl_aa_run2_econ_lib,
                                      rskmimpl_aa_curr_grp_lib=&rskmimpl_aa_curr_grp_lib,
                                      rskmimpl_aa_econ_ds=econdata_history,
                                      rskmimpl_aa_date_var=date,
                                      rskmimpl_aa_rf_to_drop_list_ds=_rskmimpl_aa_rf_to_drop,
                                      rskmimpl_aa_rf_to_swap_list_ds=_rskmimpl_aa_rf_to_swap);

    /*Swap current data*/
    %rskmimpl_aa_swap_history_current(rskmimpl_aa_run2_econ_lib=&rskmimpl_aa_run2_econ_lib,
                                      rskmimpl_aa_curr_grp_lib=&rskmimpl_aa_curr_grp_lib,
                                      rskmimpl_aa_econ_ds=econdata_current,
                                      rskmimpl_aa_date_var=,
                                      rskmimpl_aa_rf_to_drop_list_ds=_rskmimpl_aa_rf_to_drop,
                                      rskmimpl_aa_rf_to_swap_list_ds=_rskmimpl_aa_rf_to_swap);

    /*Swap scenario data*/
    %rskmimpl_aa_swap_scenario(rskmimpl_aa_run2_econ_lib=&rskmimpl_aa_run2_econ_lib,
                               rskmimpl_aa_curr_grp_lib=&rskmimpl_aa_curr_grp_lib,
                               rskmimpl_aa_econ_ds=econdata_scenarios,
                               rskmimpl_aa_rf_to_drop_list_ds=_rskmimpl_aa_rf_to_drop,
                               rskmimpl_aa_rf_to_swap_list_ds=_rskmimpl_aa_rf_to_swap);

    /*Create readvars and readrefs for grouped risk factors that are swapped in*/
    data  &rskmimpl_aa_curr_grp_lib..econdata_readvars;
        set &rskmimpl_aa_run2_econ_lib..econdata_readvars
             ( where= ( upcase(group) in (&swap_econ_vars_quoted))); 
    run;

    data &rskmimpl_aa_curr_grp_lib..econdata_readrefs;
        set &rskmimpl_aa_run2_econ_lib..econdata_readrefs;

        if _n_=1 then do;
            declare hash _rf_to_keep (dataset:"_rskmimpl_aa_rf_to_swap");
            _rf_to_keep.definekey('NAME');
            _rf_to_keep.definedata('NAME');
            _rf_to_keep.definedone();
        end;
        
        original_name=name;
        name=upcase(name);
        if _rf_to_keep.find() eq 0;
        name=original_name;
        drop original_name;
    run;
%mend rskmimpl_aa_create_scen_chg_econ;


%macro rskmimpl_aa_swap_history_current(rskmimpl_aa_run2_econ_lib=,
                                        rskmimpl_aa_curr_grp_lib=,
                                        rskmimpl_aa_econ_ds=,
                                        rskmimpl_aa_date_var=,
                                        rskmimpl_aa_rf_to_drop_list_ds=,
                                        rskmimpl_aa_rf_to_swap_list_ds=);
    %local econ_to_keep_ds;
    %if %sysfunc(exist(&rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds)) %then %do;
        %let econ_to_keep_ds=_rskmimpl_aa_curr_grp_to_keep;

        data _null_;
            set &rskmimpl_aa_rf_to_drop_list_ds end=eof;
            if _n_ = 1 then do;
                call execute ("data &econ_to_keep_ds;");
                call execute ("set &rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds(drop=");
            end;
            call execute (" "||strip(name));
            if eof then do;
                call execute (") nobs=nobs;");
                call execute ("_merge_horizon_tmp_= -nobs-1+_n_;");
                call execute ("run;");
            end;
        run;
        
        %local keep_ds_econ_vars;
        %let keep_ds_econ_vars=%rskmimpl_get_vars_from_ds(ds=&econ_to_keep_ds, exclude=&rskmimpl_aa_date_var _merge_horizon_tmp_);
        %if %sysevalf(%superq(keep_ds_econ_vars)=,boolean) %then %let econ_to_keep_ds=;
    %end;
    
    %local econ_to_swap_in_ds;
    %if %sysfunc(exist(&rskmimpl_aa_run2_econ_lib..&rskmimpl_aa_econ_ds)) %then %do;
        %let econ_to_swap_in_ds=_rskmimpl_aa_run2_econ_to_swap;

        data _null_;
            set &rskmimpl_aa_rf_to_swap_list_ds end=eof;
            if _n_ = 1 then do;
                call execute ("data &econ_to_swap_in_ds;");
                call execute ("set &rskmimpl_aa_run2_econ_lib..&rskmimpl_aa_econ_ds(keep=&rskmimpl_aa_date_var");
            end;
            call execute (" "||strip(name));
            if eof then do;
                call execute (") nobs=nobs;");
                call execute ("_merge_horizon_tmp_= -nobs-1+_n_;");
                call execute ("run;");
            end;
        run;

    %end;

    %if %length(&econ_to_keep_ds &econ_to_swap_in_ds) gt 0 %then %do;
        data &rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds;
            merge &econ_to_keep_ds &econ_to_swap_in_ds;
            by _merge_horizon_tmp_;
            drop _merge_horizon_tmp_;
        run;

        %if not %sysevalf(%superq(econ_to_keep_ds)=,boolean) %then %do;
            proc delete data = &econ_to_keep_ds;
            run;
        %end;

        %if not %sysevalf(%superq(econ_to_swap_in_ds)=,boolean) %then %do;
            proc delete data = &econ_to_swap_in_ds;
            run;
        %end;
    %end;
%mend rskmimpl_aa_swap_history_current;


%macro rskmimpl_aa_swap_scenario(rskmimpl_aa_run2_econ_lib=,
                                 rskmimpl_aa_curr_grp_lib=,
                                 rskmimpl_aa_econ_ds=,
                                 rskmimpl_aa_rf_to_drop_list_ds=,
                                 rskmimpl_aa_rf_to_swap_list_ds=);
    %local econ_to_keep_ds;
    %if %sysfunc(exist(&rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds)) %then %do;
        %let econ_to_keep_ds=_rskmimpl_aa_curr_grp_to_keep;
        data &econ_to_keep_ds;
            set &rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds;
            if 0 then set &rskmimpl_aa_rf_to_drop_list_ds;
            if _n_=1 then do;
                declare hash _rf_to_drop(dataset:"&rskmimpl_aa_rf_to_drop_list_ds");
                _rf_to_drop.definekey('name');
                _rf_to_drop.definedata('name');
                _rf_to_drop.definedone();
            end;

            _name_upcase=upcase(_name_);
            if _rf_to_drop.find(key:_name_upcase) ne 0;
            drop name _name_upcase;
        run;
    %end;
    
    %local econ_to_swap_in_ds;
    %if %sysfunc(exist(&rskmimpl_aa_run2_econ_lib..&rskmimpl_aa_econ_ds)) %then %do;
        %let econ_to_swap_in_ds=_rskmimpl_aa_run2_econ_to_swap;
        data &econ_to_swap_in_ds;
            set &rskmimpl_aa_run2_econ_lib..&rskmimpl_aa_econ_ds;
            if 0 then set &rskmimpl_aa_rf_to_swap_list_ds;
            if _n_=1 then do;
                declare hash _rf_to_swap(dataset:"&rskmimpl_aa_rf_to_swap_list_ds");
                _rf_to_swap.definekey('name');
                _rf_to_swap.definedata('name');
                _rf_to_swap.definedone();
            end;

            _name_upcase=upcase(_name_);
            if _rf_to_swap.find(key:_name_upcase) eq 0;
            drop name _name_upcase;
        run;

    %end;

    %if %length(&econ_to_keep_ds &econ_to_swap_in_ds) gt 0 %then %do;
        data &rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds;
            set &econ_to_keep_ds &econ_to_swap_in_ds;
        run;


        proc sort data=&rskmimpl_aa_curr_grp_lib..&rskmimpl_aa_econ_ds;
           by scenario_name horizon _NAME_;
        run;

        %if not %sysevalf(%superq(econ_to_keep_ds)=,boolean) %then %do;
            proc delete data = &econ_to_keep_ds;
            run;
        %end;

        %if not %sysevalf(%superq(econ_to_swap_in_ds)=,boolean) %then %do;
            proc delete data = &econ_to_swap_in_ds;
            run;
        %end;
    %end;
%mend rskmimpl_aa_swap_scenario;
